@@ -6,13 +6,18 @@
 
 import Conjure
 import Test.LeanCheck
+import Data.Function (on)
 import Data.Express hiding (height,size)
 
 data Tree  =  Leaf
            |  Node Tree Int Tree
-  deriving (Eq, Ord, Show, Read)
+  deriving (Show, Read)
 
-deriveExpress ''Tree
+instance Eq Tree where
+  (==)  =  (==) `on` inorder
+
+instance Ord Tree where
+  compare  =  compare `on` inorder
 
 unit :: Int -> Tree
 unit x  =  Node Leaf x Leaf
@@ -78,6 +83,13 @@ insert x (Node l y r)  =
   EQ -> Node l y r                -- 16
   GT -> Node l y (insert x r)     -- 22
 
+insertAlt :: Int -> Tree -> Tree
+insertAlt x Leaf          =  unit x  -- 2
+insertAlt x (Node l y r)
+  | x < y  =  Node (insert x l) y r  -- 12
+  | y < x  =  Node l y (insert x r)  -- 22
+  | otherwise  =  Node l y r         -- 25
+
 before :: Int -> Tree -> Tree
 before _ Leaf  =  Leaf
 before y (Node l x r)  =  case y `compare` x of
@@ -97,9 +109,13 @@ union t Leaf  =  t
 union t (Node l x r)  =  Node (union (before x t) l) x (union (beyond x t) r)
 
 
+deriveExpress ''Tree
+
+
 instance Listable Tree where
   tiers  =  cons0 Leaf
-        \/  cons3 Node `suchThat` ordered
+         \/ cons3 Node `suchThat` (\(Node l x r) -> (nil l || rightmost l < x)
+                                                 && (nil r || x < leftmost r))
 
 instance Name Tree where
   name _  =  "t1"
@@ -124,8 +140,7 @@ main :: IO ()
 main = do
   conjure "mem" mem
     [ con False
-    , fun "||" (||)
-    , fun "==" ((==) :: Int -> Int -> Bool)
+    , con True
     , fun "<" ((<) :: Int -> Int -> Bool)
     , guard
     ]
@@ -137,10 +152,18 @@ main = do
     , ordcase (undefined :: Bool)
     ]
 
+  -- out of reach performance-wise as well (reaching 19 but needs size 25)
+  conjure "insert" insert
+    [ fun "Node" Node
+    , fun "unit" unit
+    , guard
+    , fun "<" ((<) :: Int -> Int -> Bool)
+    , maxSize 12
+    ]
+
   -- simply out of reach performance-wise (reaching 16 but need size 22)
   conjure "insert" insert
-    [ con Leaf
-    , fun "Node" Node
+    [ fun "Node" Node
     , fun "unit" unit
     , fun "`compare`" (compare :: Int -> Int -> Ordering)
     , ordcase (undefined :: Tree)
@@ -191,6 +214,12 @@ main = do
     , maxEquationSize 7
     ]
 
+  conjure "insert" insert
+    [ fun "Node" Node
+    , fun "before" before
+    , fun "beyond" beyond
+    ]
+
   -- reachable in 55s, candidate #173109 at size 13.
   conjure "union" union
     [ con Leaf
