@@ -30,20 +30,26 @@ main  =  do
 
   -- Found!  It takes about 12 seconds to run with maxSize=8
   -- running with maxSize = 5 for faster runtime
-  conjureWith args{maxSize=5, maxConstantSize=1} "fib01" fib01
-    [ pr (0::Int)
-    , prim "dec" (subtract 1 :: Int -> Int)
-    , prim "+" ((+) :: Int -> Int -> Int)
+  conjure "fib01" fib01
+    [ con (0::Int)
+    , fun "dec" (subtract 1 :: Int -> Int)
+    , fun "+" ((+) :: Int -> Int -> Int)
+    , maxSize 5
+    , maxConstantSize 1
     ]
 
   -- It takes about 27 seconds to run with maxSize=12
   -- running with maxSize = 9 for faster runtime
-  conjureWith args{usePatterns = False, maxSize = 1, maxConstantSize=1} "fib01" fib01
-    [ pr (0::Int)
-    , prim "+" ((+) :: Int -> Int -> Int)
-    , prim "dec" (subtract 1 :: Int -> Int)
-    , prim "<=" ((<=) :: Int -> Int -> Bool)
-    , prif (undefined :: Int)
+  conjure "fib01" fib01
+    [ con (0::Int)
+    , fun "+" ((+) :: Int -> Int -> Int)
+    , fun "dec" (subtract 1 :: Int -> Int)
+    , fun "<=" ((<=) :: Int -> Int -> Bool)
+    , iif (undefined :: Int)
+
+    , singlePattern
+    , maxSize 1
+    , maxConstantSize 1
     ]
   -- expected function:
   -- fib01 x y z  =  if z <= 0                     -- 4
