@@ -97,15 +97,13 @@ instance Conjurable Tree where
 main :: IO ()
 main = do
   conjure "leftmost" leftmost
-    [ fun "undefined" (undefined :: Int)
-    , iif (undefined :: Int)
-    , fun "nil" nil
+    [ fun "nil" nil
+    , guard
     ]
 
   conjure "rightmost" rightmost
-    [ fun "undefined" (undefined :: Int)
-    , iif (undefined :: Int)
-    , fun "nil" nil
+    [ fun "nil" nil
+    , guard
     ]
 
   conjure "size" size
