@@ -41,6 +41,22 @@ module Conjure.Conjurable
   , cevl
   , Name (..)
   , Express (..)
+  , Listable (..)
+  , cons0
+  , cons1
+  , cons2
+  , cons3
+  , cons4
+  , cons5
+  , cons6
+  , (\/)
+  , reset
+  , cons7
+  , cons8
+  , cons9
+  , cons10
+  , cons11
+  , cons12
   , conjureArgumentPats
   , conjureMostGeneralCanonicalVariation
   , conjureCasesFor
@@ -149,6 +165,10 @@ class (Typeable a, Name a) => Conjurable a where
   conjureSubTypes :: a -> Reification
   conjureSubTypes _  =  id
 
+  -- | Returns an undefined value of the return value type.
+  conjureUndefined :: a -> Expr
+  conjureUndefined x  =  value "undefined" (undefined `asTypeOf` x)
+
   -- | Returns an if-function encoded as an 'Expr'.
   conjureIf :: a -> Expr
   conjureIf   =  ifFor
@@ -619,6 +639,7 @@ instance (Conjurable a, Conjurable b) => Conjurable (a -> b) where
   conjureArgumentHoles f  =  hole (argTy f) : conjureArgumentHoles (f undefined)
   conjureResultHole f  =  conjureResultHole (f undefined)
   conjureSubTypes f  =  conjureType (argTy f) . conjureType (resTy f)
+  conjureUndefined f  =  conjureUndefined (f undefined)
   conjureIf f  =  conjureIf (f undefined)
   conjureArgumentCases f  =  conjureCases (argTy f) : conjureArgumentCases (f undefined)
   conjureExpress f e
