@@ -159,7 +159,7 @@ conjureFromSpec nm p  =  conjure0 nm undefined p
 --
 --   This works like the functions 'conjure' and 'conjureFromSpec' combined.
 conjure0 :: Conjurable f => String -> f -> (f -> [Property]) -> [Ingredient] -> IO ()
-conjure0 nm f p es  =  do
+conjure0 nm f p ingredients  =  do
   -- the code section below became quite ugly with time and patches.
   -- it is still maintainable and readable as it is, but perhaps
   -- needs to be cleaned up and simplified
@@ -201,8 +201,14 @@ conjure0 nm f p es  =  do
   where
   showEq eq  =  showExpr (fst eq) ++ " == " ++ showExpr (snd eq)
   pr :: Integer -> Int -> Int -> [([Defn], [Defn])] -> IO ()
-  pr t0 n t []  =  do putWithTimeSince t0 $ "tested " ++ show t ++ " candidates"
-                      putStrLn $ nm ++ "  =  undefined  -- search exhausted\n"
+  pr t0 n t []  =  do
+    putWithTimeSince t0 $ "tested " ++ show t ++ " candidates"
+    putStrLn $ nm ++ "  =  undefined  -- search exhausted"
+    when (not carryOn) $
+      putStrLn $ "-- could not find implementation using only\n-- "
+              ++ command [showSymbol e | (e,_) <- actual ingredients]
+              ++ "\n-- consider increasing target/maxSize or refining the ingredients"
+    putStrLn ""
   pr t0 n t ((is,cs):rs)  =  do
     let nc  =  length cs
     putWithTimeSince t0 $ show nc ++ " candidates of size " ++ show n
@@ -221,21 +227,25 @@ conjure0 nm f p es  =  do
       putStrLn $ showDefn $ etaReduce $ normalizeDefn thy i
       when carryOn $ pr1 t' is (drop 1 cs'')
   rs  =  zip iss css
-  results  =  conjpure0 nm f p es
+  results  =  conjpure0 nm f p ingredients
   iss  =  implementationss results
   css  =  candidatess results
   ts   =  bindings results
   thy  =  theory results
   nRules  =  length (rules thy)
   nREs  =  length (equations thy) + nRules
+  showSymbol e
+    | isGuardSymbol e  =  "guarded equations"
+    | isIfSymbol e  =  "if-expressions"
+    | otherwise  =  showExpr e
   -- we could avoid the following as most are called once
   -- but is nice to have a summary of which settings are used
-  carryOn              =  carryOnI es
-  showTests            =  showTestsI es
-  showTheory           =  showTheoryI es
-  showPatterns         =  showPatternsI es
-  showCandidates       =  showCandidatesI es
-  showDeconstructions  =  showDeconstructionsI es
+  carryOn              =  carryOnI ingredients
+  showTests            =  showTestsI ingredients
+  showTheory           =  showTheoryI ingredients
+  showPatterns         =  showPatternsI ingredients
+  showCandidates       =  showCandidatesI ingredients
+  showDeconstructions  =  showDeconstructionsI ingredients
 
 
 -- | Results to the 'conjpure' family of functions.
@@ -267,7 +277,7 @@ conjpureFromSpec nm p  =  conjpure0 nm undefined p
 -- 'conjpure', 'conjpureFromSpec', 'conjure' and 'conjureFromSpec'
 -- all refer to this.
 conjpure0 :: Conjurable f => String -> f -> (f -> [Property]) -> [Ingredient] -> Results
-conjpure0 nm f p es  =  Results
+conjpure0 nm f p is  =  Results
   { implementationss  =  implementationsT
   , candidatess  =  candidatesT
   , bindings  =  tests
@@ -284,17 +294,17 @@ conjpure0 nm f p es  =  Results
                $  (if target > 0 then targetiers target else id)
                $  (if maxSize > 0 then take maxSize else id)
                $  candidatesTT
-  (candidatesTT, thy, patternss, deconstructions)  =  candidateDefns nm f es
+  (candidatesTT, thy, patternss, deconstructions)  =  candidateDefns nm f is
 
   test dfn  =  all (errorToFalse . deval (conjureExpress f) maxRecursions dfn False)
             $  [funToVar lhs -==- rhs | (lhs, rhs) <- tests]
   tests  =  conjureTestDefn maxTests maxSearchTests nm f
   (-==-)  =  conjureMkEquation f
-  maxTests  =  maxTestsI es
-  (target, maxSize)  =  targetAndMaxSizeI es
-  maxRecursions  =  maxRecursionsI es
-  maxSearchTests  =  maxSearchTestsI es
-  uniqueCandidates  =  uniqueCandidatesI es
+  maxTests  =  maxTestsI is
+  (target, maxSize)  =  targetAndMaxSizeI is
+  maxRecursions  =  maxRecursionsI is
+  maxSearchTests  =  maxSearchTestsI is
+  uniqueCandidates  =  uniqueCandidatesI is
 
 
 -- | Return apparently unique candidate definitions.
@@ -396,8 +406,8 @@ candidateDefnsC nm f is =
        | otherwise           =                        conjurePats maxPatternDepth es nm f
   partialDefns  =  concatMapT partialDefnsFromPats pats
   -- replaces the any guard symbol with a guard of the correct type
-  ps  =  actual is  -- extract actual ingredients/primitives from the list
-  es  =  [if isGuardSymbol e then conjureGuard f else e | (e,_) <- ps]
+  ais  =  actual is
+  es  =  [if isGuardSymbol e then conjureGuard f else e | (e,_) <- ais]
 
   eh  =  conjureResultHole f
   efxs  =  conjureVarApplication nm f
@@ -423,10 +433,10 @@ candidateDefnsC nm f is =
     keepConstant | maxConstantSize > 0  =  \e -> isFun e || isConst e || not (isGround e) || size e <= maxConstantSize
                  | otherwise            =  const True
 
-  isRedundant | adHocRedundancy  =  \e -> isRedundantDefn e || isRedundantModuloRewriting (normalize thy) e
+  isRedundant | assortedPruning  =  \e -> isRedundantDefn e || isRedundantModuloRewriting (normalize thy) e
               | otherwise        =  const False
 
-  hasRedundant | adHocRedundancy  =  hasRedundantRecursion
+  hasRedundant | assortedPruning  =  hasRedundantRecursion
                | otherwise        =  const False
 
   isNumeric  =  conjureIsNumeric f
@@ -458,6 +468,7 @@ candidateDefnsC nm f is =
       | copyBindings && isGroundPat f pat  =  [[(pat, toValPat f pat)]]
       | otherwise  =  mapT (pat,)
                    .  filterT keepB
+                   .  insemptier (conjureUndefined f)
                    .  appsWith pat
                    .  drop 1 -- this excludes the function name itself
                    $  vars pat ++ [eh | any (uncurry should) (zip aess aes)]
@@ -549,8 +560,8 @@ candidateDefnsC nm f is =
 
   thy  =  doubleCheck (===)
        .  theoryFromAtoms (===) maxEquationSize . (:[]) . nub
-       $  cjHoles (fun nm f:ps) ++ [val False, val True] ++ es
-  (===)  =  cjAreEqual (fun nm f:ps) maxTests
+       $  cjHoles (fun nm f:ais) ++ [val False, val True] ++ es
+  (===)  =  cjAreEqual (fun nm f:ais) maxTests
   isUnbreakable  =  conjureIsUnbreakable f
 
   maxTests               =  maxTestsI is
@@ -563,7 +574,7 @@ candidateDefnsC nm f is =
   requireDescent         =  requireDescentI is
   maxEarlyTests          =  maxEarlyTestsI is
   copyBindings           =  copyBindingsI is
-  adHocRedundancy        =  assortedPruningI is -- TODO: rename
+  assortedPruning        =  assortedPruningI is -- TODO: rename
   atomicNumbers          =  atomicNumbersI is
   rewriting              =  rewriteI is
 
@@ -720,3 +731,9 @@ catconMapT :: (a -> [[b]]) -> [[a]] -> [[b]]
 catconMapT f  =  foldr (\+:/) [] . map (foldr (\/) []) . mapT f
   where
   xss \+:/ yss  =  ([]:yss) \/ xss
+
+-- | If the first tier is empty,
+--   populate it with the given value.
+insemptier :: a -> [[a]] -> [[a]]
+insemptier x ([]:xss)  =  [x]:xss
+insemptier _ xss  =  xss
