@@ -39,6 +39,7 @@ module Conjure.Expr
   , isGuardSymbol
   , isGuard
   , hasGuard
+  , isIfSymbol
   , mapInnerFirstOuterLast
   , mappArgs
   , mappFun
@@ -600,6 +601,10 @@ isGuardSymbol :: Expr -> Bool
 isGuardSymbol (Value "|" _)  =  True
 isGuardSymbol _  =  False
 
+isIfSymbol :: Expr -> Bool
+isIfSymbol (Value "if" _)  =  True
+isIfSymbol _  =  False
+
 -- | Is the expression a guard application?
 isGuard :: Expr -> Bool
 isGuard (Value "|" _ :$ _ :$ _ :$ _)  =  True
